=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-02-17 13:57:25 +0000
+++ bzrlib/config.py	2006-02-21 03:29:04 +0000
@@ -76,13 +76,7 @@
 class ConfigObj(configobj.ConfigObj):
 
     def get_bool(self, section, key):
-        val = self[section][key].lower()
-        if val in ('1', 'yes', 'true', 'on'):
-            return True
-        elif val in ('0', 'no', 'false', 'off'):
-            return False
-        else:
-            raise ValueError("Value %r is not boolean" % val)
+        return self[section].as_bool(key)
 
     def get_value(self, section, name):
         # Try [] for the old DEFAULT section.
@@ -548,11 +542,10 @@
 
     def _get_config(self):
         try:
-            obj = ConfigObj(self.branch.control_files.get('branch.conf'
-                        ).readlines())
-            obj.decode('UTF-8')
+            obj = ConfigObj(self.branch.control_files.get('branch.conf'), 
+                            encoding='utf-8')
         except errors.NoSuchFile:
-            obj = ConfigObj()
+            obj = ConfigObj(encoding='utf=8')
         return obj
 
     def get_option(self, name, section=None, default=None):
@@ -583,8 +576,8 @@
                     cfg_obj[section] = {}
                     obj = cfg_obj[section]
             obj[name] = value
-            cfg_obj.encode('UTF-8')
-            out_file = StringIO(''.join([l+'\n' for l in cfg_obj.write()]))
+            out_file = StringIO()
+            cfg_obj.write(out_file)
             out_file.seek(0)
             self.branch.control_files.put('branch.conf', out_file)
         finally:

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2006-02-17 13:57:25 +0000
+++ bzrlib/tests/test_config.py	2006-02-21 03:29:04 +0000
@@ -129,6 +129,23 @@
         return self._signatures
 
 
+bool_config = """[DEFAULT]
+active = true
+inactive = false
+[UPPERCASE]
+active = True
+nonactive = False
+"""
+class TestConfigObj(TestCase):
+    def test_get_bool(self):
+        from bzrlib.config import ConfigObj
+        co = ConfigObj(StringIO(bool_config))
+        self.assertIs(co.get_bool('DEFAULT', 'active'), True)
+        self.assertIs(co.get_bool('DEFAULT', 'inactive'), False)
+        self.assertIs(co.get_bool('UPPERCASE', 'active'), True)
+        self.assertIs(co.get_bool('UPPERCASE', 'nonactive'), False)
+
+
 class TestConfig(TestCase):
 
     def test_constructs(self):

=== modified file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	2005-10-18 18:48:27 +0000
+++ bzrlib/util/configobj/configobj.py	2006-02-21 02:26:30 +0000
@@ -5,9 +5,10 @@
 #         nico AT tekNico DOT net
 
 # ConfigObj 4
+# http://www.voidspace.org.uk/python/configobj.html
 
 # Released subject to the BSD License
-# Please see http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+# Please see http://www.voidspace.org.uk/python/license.shtml
 
 # Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
 # For information about bugfixes, updates and support, please join the
@@ -15,6 +16,8 @@
 # http://lists.sourceforge.net/lists/listinfo/configobj-develop
 # Comments, suggestions and bug reports welcome.
 
+from __future__ import generators
+
 """
     >>> z = ConfigObj()
     >>> z['a'] = 'a'
@@ -37,19 +40,81 @@
 
 import os, re
 from types import StringTypes
-
-# the UTF8 BOM - from codecs module
-BOM_UTF8 = '\xef\xbb\xbf'
-
-__version__ = '4.0.0'
-
-__revision__ = '$Id: configobj.py 135 2005-10-12 14:52:28Z fuzzyman $'
+from warnings import warn
+from codecs import BOM_UTF8, BOM_UTF16, BOM_UTF16_BE, BOM_UTF16_LE
+
+# A dictionary mapping BOM to
+# the encoding to decode with, and what to set the
+# encoding attribute to.
+BOMS = {
+    BOM_UTF8: ('utf_8', None),
+    BOM_UTF16_BE: ('utf16_be', 'utf_16'),
+    BOM_UTF16_LE: ('utf16_le', 'utf_16'),
+    BOM_UTF16: ('utf_16', 'utf_16'),
+    }
+# All legal variants of the BOM codecs.
+# TODO: the list of aliases is not meant to be exhaustive, is there a
+#   better way ?
+BOM_LIST = {
+    'utf_16': 'utf_16',
+    'u16': 'utf_16',
+    'utf16': 'utf_16',
+    'utf-16': 'utf_16',
+    'utf16_be': 'utf16_be',
+    'utf_16_be': 'utf16_be',
+    'utf-16be': 'utf16_be',
+    'utf16_le': 'utf16_le',
+    'utf_16_le': 'utf16_le',
+    'utf-16le': 'utf16_le',
+    'utf_8': 'utf_8',
+    'u8': 'utf_8',
+    'utf': 'utf_8',
+    'utf8': 'utf_8',
+    'utf-8': 'utf_8',
+    }
+
+# Map of encodings to the BOM to write.
+BOM_SET = {
+    'utf_8': BOM_UTF8,
+    'utf_16': BOM_UTF16,
+    'utf16_be': BOM_UTF16_BE,
+    'utf16_le': BOM_UTF16_LE,
+    None: BOM_UTF8
+    }
+
+try:
+    from validate import VdtMissingValue
+except ImportError:
+    VdtMissingValue = None
+
+try:
+    enumerate
+except NameError:
+    def enumerate(obj):
+        """enumerate for Python 2.2."""
+        i = -1
+        for item in obj:
+            i += 1
+            yield i, item
+
+try:
+    True, False
+except NameError:
+    True, False = 1, 0
+
+
+__version__ = '4.2.0beta2'
+
+__revision__ = '$Id: configobj.py 156 2006-01-31 14:57:08Z fuzzyman $'
 
 __docformat__ = "restructuredtext en"
 
+# NOTE: Does it make sense to have the following in __all__ ?
+# NOTE: DEFAULT_INDENT_TYPE, NUM_INDENT_SPACES, MAX_INTERPOL_DEPTH
+# NOTE: If used as from configobj import...
+# NOTE: They are effectively read only
 __all__ = (
     '__version__',
-    'BOM_UTF8',
     'DEFAULT_INDENT_TYPE',
     'NUM_INDENT_SPACES',
     'MAX_INTERPOL_DEPTH',
@@ -65,6 +130,7 @@
     'MissingInterpolationOption',
     'RepeatSectionError',
     '__docformat__',
+    'flatten_errors',
 )
 
 DEFAULT_INDENT_TYPE = ' '
@@ -81,6 +147,8 @@
     'stringify': True,
     # option may be set to one of ('', ' ', '\t')
     'indent_type': None,
+    'encoding': None,
+    'default_encoding': None,
 }
 
 class ConfigObjError(SyntaxError):
@@ -196,10 +264,10 @@
 
     def __init__(self, parent, depth, main, indict=None, name=None):
         """
-        parent is the section above
-        depth is the depth level of this section
-        main is the main ConfigObj
-        indict is a dictionary to initialise the section with
+        * parent is the section above
+        * depth is the depth level of this section
+        * main is the main ConfigObj
+        * indict is a dictionary to initialise the section with
         """
         if indict is None:
             indict = {}
@@ -284,8 +352,6 @@
         """
         if not isinstance(key, StringTypes):
             raise ValueError, 'The key "%s" is not a string.' % key
-##        if self.depth is None:
-##            self.depth = 0
         # add the comment
         if not self.comments.has_key(key):
             self.comments[key] = []
@@ -346,10 +412,13 @@
             return default
 
     def update(self, indict):
-        """A version of update that uses our ``__setitem__``."""
+        """
+        A version of update that uses our ``__setitem__``.
+        """
         for entry in indict:
             self[entry] = indict[entry]
 
+
     def pop(self, key, *args):
         """ """
         val = dict.pop(self, key, *args)
@@ -456,6 +525,32 @@
             newdict[entry] = this_entry
         return newdict
 
+    def merge(self, indict):
+        """
+        A recursive update - useful for merging config files.
+        
+        >>> a = '''[section1]
+        ...     option1 = True
+        ...     [[subsection]]
+        ...     more_options = False
+        ...     # end of file'''.splitlines()
+        >>> b = '''# File is user.ini
+        ...     [section1]
+        ...     option1 = False
+        ...     # end of file'''.splitlines()
+        >>> c1 = ConfigObj(b)
+        >>> c2 = ConfigObj(a)
+        >>> c2.merge(c1)
+        >>> c2
+        {'section1': {'option1': 'False', 'subsection': {'more_options': 'False'}}}
+        """
+        for key, val in indict.items():
+            if (key in self and isinstance(self[key], dict) and
+                                isinstance(val, dict)):
+                self[key].merge(val)
+            else:   
+                self[key] = val
+
     def rename(self, oldkey, newkey):
         """
         Change a keyname to another, without changing position in sequence.
@@ -507,19 +602,49 @@
         return value when called on the whole subsection has to be discarded.
         
         See  the encode and decode methods for examples, including functions.
+        
+        .. caution::
+        
+            You can use ``walk`` to transform the names of members of a section
+            but you mustn't add or delete members.
+        
+        >>> config = '''[XXXXsection]
+        ... XXXXkey = XXXXvalue'''.splitlines()
+        >>> cfg = ConfigObj(config)
+        >>> cfg
+        {'XXXXsection': {'XXXXkey': 'XXXXvalue'}}
+        >>> def transform(section, key):
+        ...     val = section[key]
+        ...     newkey = key.replace('XXXX', 'CLIENT1')
+        ...     section.rename(key, newkey)
+        ...     if isinstance(val, (tuple, list, dict)):
+        ...         pass
+        ...     else:
+        ...         val = val.replace('XXXX', 'CLIENT1')
+        ...         section[newkey] = val
+        >>> cfg.walk(transform, call_on_sections=True)
+        {'CLIENT1section': {'CLIENT1key': None}}
+        >>> cfg
+        {'CLIENT1section': {'CLIENT1key': 'CLIENT1value'}}
         """
         out = {}
         # scalars first
-        for entry in self.scalars[:]:
+        for i in range(len(self.scalars)):
+            entry = self.scalars[i]
             try:
-                out[entry] = function(self, entry, **keywargs)
+                val = function(self, entry, **keywargs)
+                # bound again in case name has changed
+                entry = self.scalars[i]
+                out[entry] = val
             except Exception:
                 if raise_errors:
                     raise
                 else:
+                    entry = self.scalars[i]
                     out[entry] = False
         # then sections
-        for entry in self.sections[:]:
+        for i in range(len(self.sections)):
+            entry = self.sections[i]
             if call_on_sections:
                 try:
                     function(self, entry, **keywargs)
@@ -527,7 +652,10 @@
                     if raise_errors:
                         raise
                     else:
+                        entry = self.sections[i]
                         out[entry] = False
+                # bound again in case name has changed
+                entry = self.sections[i]
             # previous result is discarded
             out[entry] = self[entry].walk(
                 function,
@@ -602,6 +730,98 @@
             section[newkey] = newval
         self.walk(encode, call_on_sections=True)
 
+    def istrue(self, key):
+        """A deprecated version of ``as_bool``."""
+        warn('use of ``istrue`` is deprecated. Use ``as_bool`` method '
+                'instead.', DeprecationWarning)
+        return self.as_bool(key)
+
+    def as_bool(self, key):
+        """
+        Accepts a key as input. The corresponding value must be a string or
+        the objects (``True`` or 1) or (``False`` or 0). We allow 0 and 1 to
+        retain compatibility with Python 2.2.
+        
+        If the string is one of  ``True``, ``On``, ``Yes``, or ``1`` it returns 
+        ``True``.
+        
+        If the string is one of  ``False``, ``Off``, ``No``, or ``0`` it returns 
+        ``False``.
+        
+        ``as_bool`` is not case sensitive.
+        
+        Any other input will raise a ``ValueError``.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_bool('a')
+        Traceback (most recent call last):
+        ValueError: Value "fish" is neither True nor False
+        >>> a['b'] = 'True'
+        >>> a.as_bool('b')
+        1
+        >>> a['b'] = 'off'
+        >>> a.as_bool('b')
+        0
+        """
+        val = self[key]
+        if val == True:
+            return True
+        elif val == False:
+            return False
+        else:
+            try:
+                if not isinstance(val, StringTypes):
+                    raise KeyError
+                else:
+                    return self.main._bools[val.lower()]
+            except KeyError:
+                raise ValueError('Value "%s" is neither True nor False' % val)
+
+    def as_int(self, key):
+        """
+        A convenience method which coerces the specified value to an integer.
+        
+        If the value is an invalid literal for ``int``, a ``ValueError`` will
+        be raised.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_int('a')
+        Traceback (most recent call last):
+        ValueError: invalid literal for int(): fish
+        >>> a['b'] = '1'
+        >>> a.as_int('b')
+        1
+        >>> a['b'] = '3.2'
+        >>> a.as_int('b')
+        Traceback (most recent call last):
+        ValueError: invalid literal for int(): 3.2
+        """
+        return int(self[key])
+
+    def as_float(self, key):
+        """
+        A convenience method which coerces the specified value to a float.
+        
+        If the value is an invalid literal for ``float``, a ``ValueError`` will
+        be raised.
+        
+        >>> a = ConfigObj()
+        >>> a['a'] = 'fish'
+        >>> a.as_float('a')
+        Traceback (most recent call last):
+        ValueError: invalid literal for float(): fish
+        >>> a['b'] = '1'
+        >>> a.as_float('b')
+        1.0
+        >>> a['b'] = '3.2'
+        >>> a.as_float('b')
+        3.2000000000000002
+        """
+        return float(self[key])
+    
+
 class ConfigObj(Section):
     """
     An object to read, create, and write config files.
@@ -722,6 +942,14 @@
         '"""': (_single_line_double, _multi_line_double),
     }
 
+    # Used by the ``istrue`` Section method
+    _bools = {
+        'yes': True, 'no': False,
+        'on': True, 'off': False,
+        '1': True, '0': False,
+        'true': True, 'false': False,
+        }
+
     def __init__(self, infile=None, options=None, **kwargs):
         """
         Parse or create a config file object.
@@ -741,11 +969,13 @@
         for entry in options.keys():
             if entry not in defaults.keys():
                 raise TypeError, 'Unrecognised option "%s".' % entry
-        # TODO: check the values too
-        # add the explicit options to the defaults
+        # TODO: check the values too.
+        #
+        # Add any explicit options to the defaults
         defaults.update(options)
         #
         # initialise a few variables
+        self.filename = None
         self._errors = []
         self.raise_errors = defaults['raise_errors']
         self.interpolation = defaults['interpolation']
@@ -754,16 +984,18 @@
         self.file_error = defaults['file_error']
         self.stringify = defaults['stringify']
         self.indent_type = defaults['indent_type']
-        # used by the write method
-        self.BOM = None
+        self.encoding = defaults['encoding']
+        self.default_encoding = defaults['default_encoding']
+        self.BOM = False
+        self.newlines = None
         #
         self.initial_comment = []
         self.final_comment = []
         #
         if isinstance(infile, StringTypes):
-            self.filename = os.path.abspath(infile)
-            if os.path.isfile(self.filename):
-                infile = open(self.filename).readlines()
+            self.filename = infile
+            if os.path.isfile(infile):
+                infile = open(infile).read() or []
             elif self.file_error:
                 # raise an error if the file doesn't exist
                 raise IOError, 'Config file not found: "%s".' % self.filename
@@ -772,12 +1004,12 @@
                 if self.create_empty:
                     # this is a good test that the filename specified
                     # isn't impossible - like on a non existent device
-                    h = open(self.filename)
+                    h = open(infile, 'w')
                     h.write('')
                     h.close()
                 infile = []
         elif isinstance(infile, (list, tuple)):
-            self.filename = None
+            infile = list(infile)
         elif isinstance(infile, dict):
             # initialise self
             # the Section class handles creating subsections
@@ -786,33 +1018,37 @@
                 infile = infile.dict()
             for entry in infile:
                 self[entry] = infile[entry]
-            self.filename = None
             del self._errors
             if defaults['configspec'] is not None:
                 self._handle_configspec(defaults['configspec'])
             else:
                 self.configspec = None
             return
-        elif hasattr(infile, 'seek'):
-            # this supports StringIO instances and even file objects
-            self.filename = infile
-            infile.seek(0)
-            infile = infile.readlines()
-            self.filename.seek(0)
+        elif hasattr(infile, 'read'):
+            # This supports file like objects
+            infile = infile.read() or []
+            # needs splitting into lines - but needs doing *after* decoding
+            # in case it's not an 8 bit encoding
         else:
             raise TypeError, ('infile must be a filename,'
-                ' StringIO instance, or a file as a list.')
-        #
-        # strip trailing '\n' from lines
-        infile = [line.rstrip('\n') for line in infile]
-        #
-        # remove the UTF8 BOM if it is there
-        # FIXME: support other BOM
-        if infile and infile[0].startswith(BOM_UTF8):
-            infile[0] = infile[0][3:]
-            self.BOM = BOM_UTF8
-        else:
-            self.BOM = None
+                ' file like object, or list of lines.')
+        #
+        if infile:
+            # don't do it for the empty ConfigObj
+            infile = self._handle_bom(infile)
+            # infile is now *always* a list
+            #
+            # Set the newlines attribute (first line ending it finds)
+            # and strip trailing '\n' or '\r' from lines
+            for line in infile:
+                if (not line) or (line[-1] not in '\r\n'):
+                    continue
+                for end in ('\r\n', '\n', '\r'):
+                    if line.endswith(end):
+                        self.newlines = end
+                        break
+                break
+            infile = [line.rstrip('\r\n') for line in infile]
         #
         self._parse(infile)
         # if we had any errors, now is the time to raise them
@@ -832,6 +1068,150 @@
         else:
             self._handle_configspec(defaults['configspec'])
 
+    def _handle_bom(self, infile):
+        """
+        Handle any BOM, and decode if necessary.
+        
+        If an encoding is specified, that *must* be used - but the BOM should
+        still be removed (and the BOM attribute set).
+        
+        (If the encoding is wrongly specified, then a BOM for an alternative
+        encoding won't be discovered or removed.)
+        
+        If an encoding is not specified, UTF8 or UTF16 BOM will be detected and
+        removed. The BOM attribute will be set. UTF16 will be decoded to
+        unicode.
+        
+        NOTE: This method must not be called with an empty ``infile``.
+        
+        Specifying the *wrong* encoding is likely to cause a
+        ``UnicodeDecodeError``.
+        
+        ``infile`` must always be returned as a list of lines, but may be
+        passed in as a single string.
+        """
+        if ((self.encoding is not None) and
+            (self.encoding.lower() not in BOM_LIST)):
+            # No need to check for a BOM
+            # encoding specified doesn't have one
+            # just decode
+            return self._decode(infile, self.encoding)
+        #
+        if isinstance(infile, (list, tuple)):
+            line = infile[0]
+        else:
+            line = infile
+        if self.encoding is not None:
+            # encoding explicitly supplied
+            # And it could have an associated BOM
+            # TODO: if encoding is just UTF16 - we ought to check for both
+            # TODO: big endian and little endian versions.
+            enc = BOM_LIST[self.encoding.lower()]
+            if enc == 'utf_16':
+                # For UTF16 we try big endian and little endian
+                for BOM, (encoding, final_encoding) in BOMS.items():
+                    if not final_encoding:
+                        # skip UTF8
+                        continue
+                    if infile.startswith(BOM):
+                        ### BOM discovered
+                        ##self.BOM = True
+                        # Don't need to remove BOM
+                        return self._decode(infile, encoding)
+                #
+                # If we get this far, will *probably* raise a DecodeError
+                # As it doesn't appear to start with a BOM
+                return self._decode(infile, self.encoding)
+            #
+            # Must be UTF8
+            BOM = BOM_SET[enc]
+            if not line.startswith(BOM):
+                return self._decode(infile, self.encoding)
+            #
+            newline = line[len(BOM):]
+            #
+            # BOM removed
+            if isinstance(infile, (list, tuple)):
+                infile[0] = newline
+            else:
+                infile = newline
+            self.BOM = True
+            return self._decode(infile, self.encoding)
+        #
+        # No encoding specified - so we need to check for UTF8/UTF16
+        for BOM, (encoding, final_encoding) in BOMS.items():
+            if not line.startswith(BOM):
+                continue
+            else:
+                # BOM discovered
+                self.encoding = final_encoding
+                if not final_encoding:
+                    self.BOM = True
+                    # UTF8
+                    # remove BOM
+                    newline = line[len(BOM):]
+                    if isinstance(infile, (list, tuple)):
+                        infile[0] = newline
+                    else:
+                        infile = newline
+                    # UTF8 - don't decode
+                    if isinstance(infile, StringTypes):
+                        return infile.splitlines(True)
+                    else:
+                        return infile
+                # UTF16 - have to decode
+                return self._decode(infile, encoding)
+        #
+        # No BOM discovered and no encoding specified, just return
+        if isinstance(infile, StringTypes):
+            # infile read from a file will be a single string
+            return infile.splitlines(True)
+        else:
+            return infile
+
+    def _a_to_u(self, string):
+        """Decode ascii strings to unicode if a self.encoding is specified."""
+        if not self.encoding:
+            return string
+        else:
+            return string.decode('ascii')
+
+    def _decode(self, infile, encoding):
+        """
+        Decode infile to unicode. Using the specified encoding.
+        
+        if is a string, it also needs converting to a list.
+        """
+        if isinstance(infile, StringTypes):
+            # can't be unicode
+            # NOTE: Could raise a ``UnicodeDecodeError``
+            return infile.decode(encoding).splitlines(True)
+        for i, line in enumerate(infile):
+            if not isinstance(line, unicode):
+                # NOTE: The isinstance test here handles mixed lists of unicode/string
+                # NOTE: But the decode will break on any non-string values
+                # NOTE: Or could raise a ``UnicodeDecodeError``
+                infile[i] = line.decode(encoding)
+        return infile
+
+    def _decode_element(self, line):
+        """Decode element to unicode if necessary."""
+        if not self.encoding:
+            return line
+        if isinstance(line, str) and self.default_encoding:
+            return line.decode(self.default_encoding)
+        return line
+
+    def _str(self, value):
+        """
+        Used by ``stringify`` within validate, to turn non-string values
+        into strings.
+        """
+        if not isinstance(value, StringTypes):
+            return str(value)
+        else:
+            return value
+
     def _parse(self, infile):
         """
         Actually parse the config file
@@ -1040,7 +1420,10 @@
             # no indentation used, set the type accordingly
             self.indent_type = ''
         # preserve the final comment
-        self.final_comment = comment_list
+        if not self and not self.initial_comment:
+            self.initial_comment = comment_list
+        else:
+            self.final_comment = comment_list
 
     def _match_depth(self, sect, depth):
         """
@@ -1095,13 +1478,16 @@
         Recursively quote members of a list and return a comma joined list.
         Multiline is ``False`` for lists.
         Obey list syntax for empty and single member lists.
+        
+        If ``list_values=False`` then the value is only quoted if it contains
+        a ``\n`` (is multiline).
         """
         if isinstance(value, (list, tuple)):
             if not value:
                 return ','
             elif len(value) == 1:
                 return self._quote(value[0], multiline=False) + ','
-            return ','.join([self._quote(val, multiline=False)
+            return ', '.join([self._quote(val, multiline=False)
                 for val in value])
         if not isinstance(value, StringTypes):
             if self.stringify:
@@ -1116,13 +1502,17 @@
         tdquot = "'''%s'''"
         if not value:
             return '""'
-        if not (multiline and
+        if (not self.list_values and '\n' not in value) or not (multiline and
                 ((("'" in value) and ('"' in value)) or ('\n' in value))):
+            if not self.list_values:
+                # we don't quote if ``list_values=False``
+                quot = noquot
             # for normal values either single or double quotes will do
-            if '\n' in value:
+            elif '\n' in value:
+                # will only happen if multiline is off - e.g. '\n' in key
                 raise ConfigObjError, ('Value "%s" cannot be safely quoted.' %
                     value)
-            if ((value[0] not in wspace_plus) and
+            elif ((value[0] not in wspace_plus) and
                     (value[-1] not in wspace_plus) and
                     (',' not in value)):
                 quot = noquot
@@ -1244,8 +1634,8 @@
             if mat is None:
                 raise SyntaxError
             (value, comment) = mat.groups()
-            # FIXME: unquoting here can be a source of error
-            return (self._unquote(value), comment)
+            # NOTE: we don't unquote here
+            return (value, comment)
         mat = self._valueexp.match(value)
         if mat is None:
             # the value is badly constructed, probably badly quoted,
@@ -1392,20 +1782,22 @@
 
     def _write_line(self, indent_string, entry, this_entry, comment):
         """Write an individual line, for the write method"""
-        return '%s%s = %s%s' % (
+        # NOTE: the calls to self._quote here handles non-StringType values.
+        return '%s%s%s%s%s' % (
             indent_string,
-            self._quote(entry, multiline=False),
-            self._quote(this_entry),
-            comment)
+            self._decode_element(self._quote(entry, multiline=False)),
+            self._a_to_u(' = '),
+            self._decode_element(self._quote(this_entry)),
+            self._decode_element(comment))
 
     def _write_marker(self, indent_string, depth, entry, comment):
         """Write a section marker line"""
         return '%s%s%s%s%s' % (
             indent_string,
-            '[' * depth,
-            self._quote(entry, multiline=False),
-            ']' * depth,
-            comment)
+            self._a_to_u('[' * depth),
+            self._quote(self._decode_element(entry), multiline=False),
+            self._a_to_u(']' * depth),
+            self._decode_element(comment))
 
     def _handle_comment(self, comment):
         """
@@ -1437,11 +1829,11 @@
         if not comment:
             return ''
         if self.indent_type == '\t':
-            start = '\t'
+            start = self._a_to_u('\t')
         else:
-            start = ' ' * NUM_INDENT_SPACES
+            start = self._a_to_u(' ' * NUM_INDENT_SPACES)
         if not comment.startswith('#'):
-            start += '# '
+            start += _a_to_u('# ')
         return (start + comment)
 
     def _compute_indent_string(self, depth):
@@ -1459,7 +1851,7 @@
 
     # Public methods
 
-    def write(self, section=None):
+    def write(self, outfile=None, section=None):
         """
         Write the current ConfigObj as a file
         
@@ -1488,33 +1880,34 @@
         >>> a.write()
         ['a = %(a)s', '[DEFAULT]', 'a = fish']
         """
-        int_val = 'test'
         if self.indent_type is None:
             # this can be true if initialised from a dictionary
             self.indent_type = DEFAULT_INDENT_TYPE
         #
         out = []
-        return_list = True
+        cs = self._a_to_u('#')
+        csp = self._a_to_u('# ')
         if section is None:
             int_val = self.interpolation
             self.interpolation = False
             section = self
-            return_list = False
             for line in self.initial_comment:
+                line = self._decode_element(line)
                 stripped_line = line.strip()
-                if stripped_line and not stripped_line.startswith('#'):
-                    line = '# ' + line
+                if stripped_line and not stripped_line.startswith(cs):
+                    line = csp + line
                 out.append(line)
         #
-        indent_string = self._compute_indent_string(section.depth)
+        indent_string = self._a_to_u(
+            self._compute_indent_string(section.depth))
         for entry in (section.scalars + section.sections):
             if entry in section.defaults:
                 # don't write out default values
                 continue
             for comment_line in section.comments[entry]:
-                comment_line = comment_line.lstrip()
-                if comment_line and not comment_line.startswith('#'):
-                    comment_line = '#' + comment_line
+                comment_line = self._decode_element(comment_line.lstrip())
+                if comment_line and not comment_line.startswith(cs):
+                    comment_line = csp + comment_line
                 out.append(indent_string + comment_line)
             this_entry = section[entry]
             comment = self._handle_comment(section.inline_comments[entry])
@@ -1526,7 +1919,7 @@
                     this_entry.depth,
                     entry,
                     comment))
-                out.extend(self.write(this_entry))
+                out.extend(self.write(section=this_entry))
             else:
                 out.append(self._write_line(
                     indent_string,
@@ -1534,32 +1927,49 @@
                     this_entry,
                     comment))
         #
-        if not return_list:
+        if section is self:
             for line in self.final_comment:
+                line = self._decode_element(line)
                 stripped_line = line.strip()
-                if stripped_line and not stripped_line.startswith('#'):
-                    line = '# ' + line
+                if stripped_line and not stripped_line.startswith(cs):
+                    line = csp + line
                 out.append(line)
-        #
-        if int_val != 'test':
             self.interpolation = int_val
         #
-        if (return_list) or (self.filename is None):
-            return out
-        #
-        if isinstance(self.filename, StringTypes):
+        if section is not self:
+            return out
+        #
+        if (self.filename is None) and (outfile is None):
+            # output a list of lines
+            # might need to encode
+            # NOTE: This will *screw* UTF16, each line will start with the BOM
+            if self.encoding:
+                out = [l.encode(self.encoding) for l in out]
+            if (self.BOM and ((self.encoding is None) or
+                (BOM_LIST.get(self.encoding.lower()) == 'utf_8'))):
+                # Add the UTF8 BOM
+                if not out:
+                    out.append('')
+                out[0] = BOM_UTF8 + out[0]
+            return out
+        #
+        # Turn the list to a string, joined with correct newlines
+        output = (self._a_to_u(self.newlines or os.linesep)
+            ).join(out)
+        if self.encoding:
+            output = output.encode(self.encoding)
+        if (self.BOM and ((self.encoding is None) or
+            (BOM_LIST.get(self.encoding.lower()) == 'utf_8'))):
+            # Add the UTF8 BOM
+            output = BOM_UTF8 + output
+        if outfile is not None:
+            outfile.write(output)
+        else:
             h = open(self.filename, 'w')
-            h.write(self.BOM or '')
-            h.write('\n'.join(out))
+            h.write(output)
             h.close()
-        else:
-            self.filename.seek(0)
-            self.filename.write(self.BOM or '')
-            self.filename.write('\n'.join(out))
-            # if we have a stored file object (or StringIO)
-            # we *don't* close it
 
-    def validate(self, validator, section=None):
+    def validate(self, validator, preserve_errors=False, section=None):
         """
         Test the ConfigObj against a configspec.
         
@@ -1581,10 +1991,23 @@
         In addition, it converts the values from strings to their native
         types if their checks pass (and ``stringify`` is set).
         
+        If ``preserve_errors`` is ``True`` (``False`` is default) then instead
+        of a marking a fail with a ``False``, it will preserve the actual
+        exception object. This can contain info about the reason for failure.
+        For example the ``VdtValueTooSmallError`` indeicates that the value
+        supplied was too small. If a value (or section) is missing it will
+        still be marked as ``False``.
+        
+        You must have the validate module to use ``preserve_errors=True``.
+        
+        You can then use the ``flatten_errors`` function to turn your nested
+        results dictionary into a flattened list of failures - useful for
+        displaying meaningful error messages.
+        
         >>> try:
         ...     from validate import Validator
         ... except ImportError:
-        ...     print >> sys.stderr, 'Cannot import the Validator object, skipping the realted tests'
+        ...     print >> sys.stderr, 'Cannot import the Validator object, skipping the related tests'
         ... else:
         ...     config = '''
         ...     test1=40
@@ -1603,20 +2026,20 @@
         ...             test4=5.0
         ... '''.split('\\n')
         ...     configspec = '''
-        ...     test1='integer(30,50)'
-        ...     test2='string'
-        ...     test3='integer'
-        ...     test4='float(6.0)'
+        ...     test1= integer(30,50)
+        ...     test2= string
+        ...     test3=integer
+        ...     test4=float(6.0)
         ...     [section ]
-        ...         test1='integer(30,50)'
-        ...         test2='string'
-        ...         test3='integer'
-        ...         test4='float(6.0)'
+        ...         test1=integer(30,50)
+        ...         test2=string
+        ...         test3=integer
+        ...         test4=float(6.0)
         ...         [[sub section]]
-        ...             test1='integer(30,50)'
-        ...             test2='string'
-        ...             test3='integer'
-        ...             test4='float(6.0)'
+        ...             test1=integer(30,50)
+        ...             test2=string
+        ...             test3=integer
+        ...             test4=float(6.0)
         ...     '''.split('\\n')
         ...     val = Validator()
         ...     c1 = ConfigObj(config, configspec=configspec)
@@ -1670,20 +2093,20 @@
         >>> val_res == {'key2': True, 'section': True, 'key': False}
         1
         >>> configspec = '''
-        ...     test1='integer(30,50, default=40)'
-        ...     test2='string(default="hello")'
-        ...     test3='integer(default=3)'
-        ...     test4='float(6.0, default=6.0)'
+        ...     test1=integer(30,50, default=40)
+        ...     test2=string(default="hello")
+        ...     test3=integer(default=3)
+        ...     test4=float(6.0, default=6.0)
         ...     [section ]
-        ...         test1='integer(30,50, default=40)'
-        ...         test2='string(default="hello")'
-        ...         test3='integer(default=3)'
-        ...         test4='float(6.0, default=6.0)'
+        ...         test1=integer(30,50, default=40)
+        ...         test2=string(default="hello")
+        ...         test3=integer(default=3)
+        ...         test4=float(6.0, default=6.0)
         ...         [[sub section]]
-        ...             test1='integer(30,50, default=40)'
-        ...             test2='string(default="hello")'
-        ...             test3='integer(default=3)'
-        ...             test4='float(6.0, default=6.0)'
+        ...             test1=integer(30,50, default=40)
+        ...             test2=string(default="hello")
+        ...             test3=integer(default=3)
+        ...             test4=float(6.0, default=6.0)
         ...     '''.split('\\n')
         >>> default_test = ConfigObj(['test1=30'], configspec=configspec)
         >>> default_test
@@ -1926,6 +2349,7 @@
         1
         
         Test that interpolation is preserved for validated string values.
+        Also check that interpolation works in configspecs.
         >>> t = ConfigObj()
         >>> t['DEFAULT'] = {}
         >>> t['DEFAULT']['test'] = 'a'
@@ -1939,6 +2363,16 @@
         >>> t.interpolation = False
         >>> t
         {'test': '%(test)s', 'DEFAULT': {'test': 'a'}}
+        >>> specs = [
+        ...    'interpolated string  = string(default="fuzzy-%(man)s")',
+        ...    '[DEFAULT]',
+        ...    'man = wuzzy',
+        ...    ]
+        >>> c = ConfigObj(configspec=specs)
+        >>> c.validate(v)
+        1
+        >>> c['interpolated string']
+        'fuzzy-wuzzy'
         
         FIXME: Above tests will fail if we couldn't import Validator (the ones
         that don't raise errors will produce different output and still fail as
@@ -1947,6 +2381,9 @@
         if section is None:
             if self.configspec is None:
                 raise ValueError, 'No configspec supplied.'
+            if preserve_errors:
+                if VdtMissingValue is None:
+                    raise ImportError('Missing validate module.')
             section = self
         #
         spec_section = section.configspec
@@ -1974,13 +2411,16 @@
             try:
                 check = validator.check(spec_section[entry],
                                         val,
-                                        missing=missing)
-            except validator.baseErrorClass:
-                out[entry] = False
+                                        missing=missing
+                                        )
+            except validator.baseErrorClass, e:
+                if not preserve_errors or isinstance(e, VdtMissingValue):
+                    out[entry] = False
+                else:
+                    # preserve the error
+                    out[entry] = e
+                    ret_false = False
                 ret_true = False
-            # MIKE: we want to raise all other exceptions, not just print ?
-##            except Exception, err:
-##                print err
             else:
                 ret_false = False
                 out[entry] = True
@@ -1990,19 +2430,23 @@
                     if not self.stringify:
                         if isinstance(check, (list, tuple)):
                             # preserve lists
-                            check = [str(item) for item in check]
+                            check = [self._str(item) for item in check]
                         elif missing and check is None:
                             # convert the None from a default to a ''
                             check = ''
                         else:
-                            check = str(check)
+                            check = self._str(check)
                     if (check != val) or missing:
                         section[entry] = check
                 if missing and entry not in section.defaults:
                     section.defaults.append(entry)
         #
+        # FIXME: Will this miss missing sections ?
         for entry in section.sections:
-            check = self.validate(validator, section[entry])
+            if section is self and entry == 'DEFAULT':
+                continue
+            check = self.validate(validator, preserve_errors=preserve_errors,
+                section=section[entry])
             out[entry] = check
             if check == False:
                 ret_true = False
@@ -2080,6 +2524,131 @@
             raise self.baseErrorClass
         return member
 
+# Check / processing functions for options
+def flatten_errors(cfg, res, levels=None, results=None):
+    """
+    An example function that will turn a nested dictionary of results
+    (as returned by ``ConfigObj.validate``) into a flat list.
+    
+    ``cfg`` is the ConfigObj instance being checked, ``res`` is the results
+    dictionary returned by ``validate``.
+    
+    (This is a recursive function, so you shouldn't use the ``levels`` or
+    ``results`` arguments - they are used by the function.
+    
+    Returns a list of keys that failed. Each member of the list is a tuple :
+    ::
+    
+        ([list of sections...], key, result)
+    
+    If ``validate`` was called with ``preserve_errors=False`` (the default)
+    then ``result`` will always be ``False``.
+
+    *list of sections* is a flattened list of sections that the key was found
+    in.
+    
+    If the section was missing then key will be ``None``.
+    
+    If the value (or section) was missing then ``result`` will be ``False``.
+    
+    If ``validate`` was called with ``preserve_errors=True`` and a value
+    was present, but failed the check, then ``result`` will be the exception
+    object returned. You can use this as a string that describes the failure.
+    
+    For example *The value "3" is of the wrong type*.
+    
+    # FIXME: is the ordering of the output arbitrary ?
+    >>> import validate
+    >>> vtor = validate.Validator()
+    >>> my_ini = '''
+    ...     option1 = True
+    ...     [section1]
+    ...     option1 = True
+    ...     [section2]
+    ...     another_option = Probably
+    ...     [section3]
+    ...     another_option = True
+    ...     [[section3b]]
+    ...     value = 3
+    ...     value2 = a
+    ...     value3 = 11
+    ...     '''
+    >>> my_cfg = '''
+    ...     option1 = boolean()
+    ...     option2 = boolean()
+    ...     option3 = boolean(default=Bad_value)
+    ...     [section1]
+    ...     option1 = boolean()
+    ...     option2 = boolean()
+    ...     option3 = boolean(default=Bad_value)
+    ...     [section2]
+    ...     another_option = boolean()
+    ...     [section3]
+    ...     another_option = boolean()
+    ...     [[section3b]]
+    ...     value = integer
+    ...     value2 = integer
+    ...     value3 = integer(0, 10)
+    ...         [[[section3b-sub]]]
+    ...         value = string
+    ...     [section4]
+    ...     another_option = boolean()
+    ...     '''
+    >>> cs = my_cfg.split('\\n')
+    >>> ini = my_ini.split('\\n')
+    >>> cfg = ConfigObj(ini, configspec=cs)
+    >>> res = cfg.validate(vtor, preserve_errors=True)
+    >>> errors = []
+    >>> for entry in flatten_errors(cfg, res):
+    ...     section_list, key, error = entry
+    ...     section_list.insert(0, '[root]')
+    ...     if key is not None:
+    ...        section_list.append(key)
+    ...     else:
+    ...         section_list.append('[missing]')
+    ...     section_string = ', '.join(section_list)
+    ...     errors.append((section_string, ' = ', error))
+    >>> errors.sort()
+    >>> for entry in errors:
+    ...     print entry[0], entry[1], (entry[2] or 0)
+    [root], option2  =  0
+    [root], option3  =  the value "Bad_value" is of the wrong type.
+    [root], section1, option2  =  0
+    [root], section1, option3  =  the value "Bad_value" is of the wrong type.
+    [root], section2, another_option  =  the value "Probably" is of the wrong type.
+    [root], section3, section3b, section3b-sub, [missing]  =  0
+    [root], section3, section3b, value2  =  the value "a" is of the wrong type.
+    [root], section3, section3b, value3  =  the value "11" is too big.
+    [root], section4, [missing]  =  0
+    """
+    if levels is None:
+        # first time called
+        levels = []
+        results = []
+    if res is True:
+        return results
+    if res is False:
+        results.append((levels[:], None, False))
+        if levels:
+            levels.pop()
+        return results
+    for (key, val) in res.items():
+        if val == True:
+            continue
+        if isinstance(cfg.get(key), dict):
+            # Go down one level
+            levels.append(key)
+            flatten_errors(cfg[key], val, levels, results)
+            continue
+        results.append((levels[:], key, val))
+    #
+    # Go up one level
+    if levels:
+        levels.pop()
+    #
+    return results
+
+
 # FIXME: test error code for badly built multiline values
 # FIXME: test handling of StringIO
 # FIXME: test interpolation with writing
@@ -2146,6 +2715,147 @@
     >>> t2.inline_comments['b'] = ''
     >>> del t2['a']
     >>> assert t2.write() == ['','b = b', '']
+    
+    # Test ``list_values=False`` stuff
+    >>> c = '''
+    ...     key1 = no quotes
+    ...     key2 = 'single quotes'
+    ...     key3 = "double quotes"
+    ...     key4 = "list", 'with', several, "quotes"
+    ...     '''
+    >>> cfg = ConfigObj(c.splitlines(), list_values=False)
+    >>> cfg == {'key1': 'no quotes', 'key2': "'single quotes'", 
+    ... 'key3': '"double quotes"', 
+    ... 'key4': '"list", \\'with\\', several, "quotes"'
+    ... }
+    1
+    >>> cfg = ConfigObj(list_values=False)
+    >>> cfg['key1'] = 'Multiline\\nValue'
+    >>> cfg['key2'] = '''"Value" with 'quotes' !'''
+    >>> cfg.write()
+    ["key1 = '''Multiline\\nValue'''", 'key2 = "Value" with \\'quotes\\' !']
+    >>> cfg.list_values = True
+    >>> cfg.write() == ["key1 = '''Multiline\\nValue'''",
+    ... 'key2 = \\'\\'\\'"Value" with \\'quotes\\' !\\'\\'\\'']
+    1
+    
+    Test flatten_errors:
+    
+    >>> from validate import Validator, VdtValueTooSmallError
+    >>> config = '''
+    ...     test1=40
+    ...     test2=hello
+    ...     test3=3
+    ...     test4=5.0
+    ...     [section]
+    ...         test1=40
+    ...         test2=hello
+    ...         test3=3
+    ...         test4=5.0
+    ...         [[sub section]]
+    ...             test1=40
+    ...             test2=hello
+    ...             test3=3
+    ...             test4=5.0
+    ... '''.split('\\n')
+    >>> configspec = '''
+    ...     test1= integer(30,50)
+    ...     test2= string
+    ...     test3=integer
+    ...     test4=float(6.0)
+    ...     [section ]
+    ...         test1=integer(30,50)
+    ...         test2=string
+    ...         test3=integer
+    ...         test4=float(6.0)
+    ...         [[sub section]]
+    ...             test1=integer(30,50)
+    ...             test2=string
+    ...             test3=integer
+    ...             test4=float(6.0)
+    ...     '''.split('\\n')
+    >>> val = Validator()
+    >>> c1 = ConfigObj(config, configspec=configspec)
+    >>> res = c1.validate(val)
+    >>> flatten_errors(c1, res) == [([], 'test4', False), (['section', 
+    ...     'sub section'], 'test4', False), (['section'], 'test4', False)]
+    True
+    >>> res = c1.validate(val, preserve_errors=True)
+    >>> check = flatten_errors(c1, res)
+    >>> check[0][:2]
+    ([], 'test4')
+    >>> check[1][:2]
+    (['section', 'sub section'], 'test4')
+    >>> check[2][:2]
+    (['section'], 'test4')
+    >>> for entry in check:
+    ...     isinstance(entry[2], VdtValueTooSmallError)
+    ...     print str(entry[2])
+    True
+    the value "5.0" is too small.
+    True
+    the value "5.0" is too small.
+    True
+    the value "5.0" is too small.
+    
+    Test unicode handling, BOM, write witha file like object and line endings :
+    >>> u_base = '''
+    ... # initial comment
+    ...     # inital comment 2
+    ... 
+    ... test1 = some value
+    ... # comment
+    ... test2 = another value    # inline comment
+    ... # section comment
+    ... [section]    # inline comment
+    ...     test = test    # another inline comment
+    ...     test2 = test2
+    ... 
+    ... # final comment
+    ... # final comment2
+    ... '''
+    >>> u = u_base.encode('utf_8').splitlines(True)
+    >>> u[0] = BOM_UTF8 + u[0]
+    >>> uc = ConfigObj(u)
+    >>> uc.encoding = None
+    >>> uc.BOM == True
+    1
+    >>> uc == {'test1': 'some value', 'test2': 'another value',
+    ... 'section': {'test': 'test', 'test2': 'test2'}}
+    1
+    >>> uc = ConfigObj(u, encoding='utf_8', default_encoding='latin-1')
+    >>> uc.BOM
+    1
+    >>> isinstance(uc['test1'], unicode)
+    1
+    >>> uc.encoding
+    'utf_8'
+    >>> uc.newlines
+    '\\n'
+    >>> uc['latin1'] = "This costs lot's of "
+    >>> a_list = uc.write()
+    >>> len(a_list)
+    15
+    >>> isinstance(a_list[0], str)
+    1
+    >>> a_list[0].startswith(BOM_UTF8)
+    1
+    >>> u = u_base.replace('\\n', '\\r\\n').encode('utf_8').splitlines(True)
+    >>> uc = ConfigObj(u)
+    >>> uc.newlines
+    '\\r\\n'
+    >>> uc.newlines = '\\r'
+    >>> from cStringIO import StringIO
+    >>> file_like = StringIO()
+    >>> uc.write(file_like)
+    >>> file_like.seek(0)
+    >>> uc2 = ConfigObj(file_like)
+    >>> uc2 == uc
+    1
+    >>> uc2.filename == None
+    1
+    >>> uc2.newlines == '\\r'
+    1
     """
 
 if __name__ == '__main__':
@@ -2235,16 +2945,35 @@
     BUGS
     ====
     
-    With list values off, ConfigObj can incorrectly unquote values. (This makes
-    it impossible to use listquote to handle your list values for you - for
-    nested lists. Not handling quotes at all would be better for this)
+    None known.
     
     TODO
     ====
     
+    Better support for configuration from multiple files, including tracking
+    *where* the original file came from and writing changes to the correct
+    file.
+    
+    
+    Make ``newline`` an option (as well as an attribute) ?
+    
+    ``UTF16`` encoded files, when returned as a list of lines, will have the
+    BOM at the start of every line. Should this be removed from all but the
+    first line ?
+    
+    Option to set warning type for unicode decode ? (Defaults to strict).
+    
     A method to optionally remove uniform indentation from multiline values.
     (do as an example of using ``walk`` - along with string-escape)
     
+    Should the results dictionary from validate be an ordered dictionary if
+    `odict <http://www.voidspace.org.uk/python/odict.html>`_ is available ?
+    
+    Implement a better ``__repr__`` ? (``ConfigObj({})``)
+    
+    Implement some of the sequence methods (which include slicing) from the
+    newer ``odict`` ?
+    
     INCOMPATIBLE CHANGES
     ====================
     
@@ -2304,9 +3033,15 @@
     ISSUES
     ======
     
+    ``validate`` doesn't report *extra* values or sections.
+    
     You can't have a keyword with the same name as a section (in the same
     section). They are both dictionary keys - so they would overlap.
     
+    ConfigObj doesn't quote and unquote values if ``list_values=False``.
+    This means that leading or trailing whitespace in values will be lost when
+    writing. (Unless you manually quote).
+    
     Interpolation checks first the 'DEFAULT' subsection of the current
     section, next it checks the 'DEFAULT' section of the parent section,
     last it checks the 'DEFAULT' section of the main section.
@@ -2325,13 +3060,6 @@
     Does it matter that we don't support the ':' divider, which is supported
     by ``ConfigParser`` ?
     
-    Following error with "list_values=False" : ::
-    
-        >>> a = ["a='hello', 'goodbye'"]
-        >>>
-        >>> c(a, list_values=False)
-        {'a': "hello', 'goodbye"}
-    
     The regular expression correctly removes the value -
     ``"'hello', 'goodbye'"`` and then unquote just removes the front and
     back quotes (called from ``_handle_value``). What should we do ??
@@ -2345,6 +3073,7 @@
     If the value is unchanged by validation (it's a string) - but other types
     will be.
     
+    
     List Value Syntax
     =================
     
@@ -2375,11 +3104,99 @@
     CHANGELOG
     =========
     
+    2006/02/04
+    ----------
+    
+    Removed ``BOM_UTF8`` from ``__all__``.
+    
+    The ``BOM`` attribute has become a boolean. (Defaults to ``False``.) It is
+    *only* ``True`` for the ``UTF16`` encoding.
+    
+    File like objects no longer need a ``seek`` attribute.
+    
+    ConfigObj no longer keeps a reference to file like objects. Instead the
+    ``write`` method takes a file like object as an optional argument. (Which
+    will be used in preference of the ``filename`` attribute if htat exists as
+    well.)
+    
+    Full unicode support added. New options/attributes ``encoding``,
+    ``default_encoding``.
+    
+    utf16 files decoded to unicode.
+    
+    If ``BOM`` is ``True``, but no encoding specified, then the utf8 BOM is
+    written out at the start of the file. (It will normally only be ``True`` if
+    the utf8 BOM was found when the file was read.)
+    
+    File paths are *not* converted to absolute paths, relative paths will
+    remain relative as the ``filename`` attribute.
+    
+    Fixed bug where ``final_comment`` wasn't returned if ``write`` is returning
+    a list of lines.
+    
+    2006/01/31
+    ----------
+    
+    Added ``True``, ``False``, and ``enumerate`` if they are not defined.
+    (``True`` and ``False`` are needed for *early* versions of Python 2.2,
+    ``enumerate`` is needed for all versions ofPython 2.2)
+    
+    Deprecated ``istrue``, replaced it with ``as_bool``.
+    
+    Added ``as_int`` and ``as_float``.
+    
+    utf8 and utf16 BOM handled in an endian agnostic way.
+    
+    2005/12/14
+    ----------
+    
+    Validation no longer done on the 'DEFAULT' section (only in the root
+    level). This allows interpolation in configspecs.
+    
+    Change in validation syntax implemented in validate 0.2.1
+    
+    4.1.0
+    
+    2005/12/10
+    ----------
+    
+    Added ``merge``, a recursive update.
+    
+    Added ``preserve_errors`` to ``validate`` and the ``flatten_errors``
+    example function.
+    
+    Thanks to Matthew Brett for suggestions and helping me iron out bugs.
+    
+    Fixed bug where a config file is *all* comment, the comment will now be
+    ``initial_comment`` rather than ``final_comment``.
+    
+    2005/12/02
+    ----------
+    
+    Fixed bug in ``create_empty``. Thanks to Paul Jimenez for the report.
+    
+    2005/11/04
+    ----------
+    
+    Fixed bug in ``Section.walk`` when transforming names as well as values.
+    
+    Added the ``istrue`` method. (Fetches the boolean equivalent of a string
+    value).
+    
+    Fixed ``list_values=False`` - they are now only quoted/unquoted if they
+    are multiline values.
+    
+    List values are written as ``item, item`` rather than ``item,item``.
+    
+    4.0.1
+    
     2005/10/09
     ----------
     
     Fixed typo in ``write`` method. (Testing for the wrong value when resetting
     ``interpolation``).
+
+    4.0.0 Final
     
     2005/09/16
     ----------
@@ -2410,7 +3227,7 @@
     2005/09/03
     ----------
     
-    Fixed bug in ``Section__delitem__`` oops.
+    Fixed bug in ``Section.__delitem__`` oops.
     
     2005/08/28
     ----------

=== modified file 'bzrlib/util/configobj/docs/configobj.txt'
--- bzrlib/util/configobj/docs/configobj.txt	2005-10-18 18:48:27 +0000
+++ bzrlib/util/configobj/docs/configobj.txt	2006-02-21 02:26:30 +0000
@@ -7,8 +7,8 @@
 ----------------------------------------
 
 :Authors: Michael Foord, Nicola Larosa
-:Version: ConfigObj 4.0.0
-:Date: 2005/10/17
+:Version: ConfigObj 4.2.0
+:Date: 2006/02/16
 :Homepage: `ConfigObj Homepage`_
 :Sourceforge: Sourceforge_
 :License: `BSD License`_
@@ -33,44 +33,68 @@
 straightforward programmer's interface and a simple syntax for config files.
 It has lots of other features though :
 
+.. sidebar:: ConfigObj in Bazaar
+
+    **ConfigObj** is now used as the config file parser for `Bazaar <http://bazaar-ng.org>`_.
+    
+    Bazaar is *the* Python  distributed {acro;VCS;Version Control System}. 
+    ConfigObj is used to read ``bazaar.conf`` and ``branches.conf``.
+    
+    Other projects that use **ConfigObj** include :
+    
+    * `Planet Plus <http://planetplus.python-hosting.com/>`_
+    
+        A new web application version of `Planet <http://www.planetplanet.org/>`_, 
+        the web aggregator.
+        
+    * `NeuroImaging in Python <http://projects.scipy.org/neuroimaging/ni/wiki>`_
+    
+        BrainSTAT is a project with the ultimate goal to produce a
+        platform-independent python environment for the analysis of brain
+        imaging data.
+        
+    * `Gruik <http://www.tracos.org/gruik/wiki>`_
+    
+        Gruik is a free software network packet sniffer.
+
 * Nested sections (subsections), to any level
 * List values
 * Multiple line values
 * String interpolation (substitution)
 * Integrated with a powerful validation system
 
-    * including automatic type checking/conversion
-    * repeated sections
-    * and allowing default values
+    - including automatic type checking/conversion
+    - repeated sections
+    - and allowing default values
 
 * All comments in the file are preserved
 * The order of keys/sections is preserved
 * No external dependencies
-
-ConfigObj 4 is a complete rewrite of ConfigObj. A great deal has been
-simplified and improved [#]_ since ConfigObj 3. If you have used ConfigObj
-before then you need to read the section on `backwards compatibility`_.
-
-ConfigObj now has a barrage [#]_ of doctests built into it, testing almost every
-feature. Run ``python configobj.py -v`` to see them in action. Despite the tests,
-ConfigObj 4 is actually smaller than version 3 and has no external dependencies.
+* Full unicode support.
+
+ConfigObj 4 is a complete rewrite of ConfigObj. ConfigObj now has a barrage [#]_
+of doctests built into it, testing almost every feature. Run ``python configobj.py -v``
+to see them in action. Despite the tests, ConfigObj 4 is smaller than version 3
+and has no external dependencies.
 
 For support and bug reports please use the ConfigObj `Mailing List`_.
 
 .. hint::
 
-    There is an article on using `ConfigObj for Data Persistence`_.
+    ConfigObj provides a convenient API for storing all sorts of data, not just
+    config files. There is an article on using `ConfigObj for Data Persistence`_.
     
     The code from that article is available as ConfigPersist.py_.
 
 .. _ConfigObj for Data Persistence: http://www.voidspace.org.uk/python/articles/configobj_for_data_persistence.shtml
 .. _ConfigPersist.py: http://www.voidspace.org.uk/python/configpersist.html
 
+
 Downloading
 ===========
 
-The current version is **4.0.0**, dated 17th October 2005. ConfigObj 4 is
-now stable. We may still expect to pick up a few bugs along the way though [#]_.
+The current version is **4.2.0**, dated 16th February 2006. ConfigObj 4 is
+now stable. We still expect to pick up a few bugs along the way though [#]_.
 {sm;:-)}
 
 You can get ConfigObj in the following ways :
@@ -112,7 +136,7 @@
 Projects`_.
 
 .. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
-.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0.zip
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.2.0.zip
 .. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
 .. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
 .. _this document:
@@ -123,6 +147,7 @@
 .. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
 .. _Voidspace Python Projects: http://www.voidspace.org.uk/python
 
+
 Getting Started
 ===============
 
@@ -267,6 +292,7 @@
     # The 'final_comment'
     # Which also may be several lines
 
+
 ConfigObj specifications
 ========================
 
@@ -291,14 +317,11 @@
   the options_ ``file_error`` and ``create_empty``. The filename will be
   preserved as the ``filename`` attribute. This can be changed at any time.
 
-* A list of lines. Any trailing ``\n`` will be removed from the lines. The
+* A list of lines. Any trailing newlines will be removed from the lines. The
   ``filename`` attribute of your ConfigObj will be ``None``.
 
-* A ``StringIO`` instance or file object, or any object with ``seek`` and
-  ``read`` methods. The object you pass in will be preserved as the
-  ``filename`` attribute of your ConfigObj. If it has a ``write`` method [#]_
-  then you can use the ConfigObj ``write`` method. Note that a file object
-  passed in won't have its ``close`` method called by ConfigObj.
+* A ``StringIO`` instance or file object, or any object with a ``read`` method.
+  The ``filename`` attribute of your ConfigObj will be ``None`` [#]_.
 
 * A dictionary. You can initialise a ConfigObj from a dictionary [#]_. The
   ``filename`` attribute of your ConfigObj will be ``None``. All keys must be
@@ -329,8 +352,12 @@
 
     If ``True`` (the default) then list values are possible. If ``False``, the
     values are not parsed for lists.
+	
+	If ``list_values = False`` then single line values are not quoted or
+	unqouted when reading and writing.
 
-    Altering this value after initial parsing has no effect.
+    Changing this value affects whether single line values will be quoted or 
+    not when writing.
 
 * 'create_empty': ``False``
 
@@ -402,6 +429,46 @@
     If this option *is* specified, the option value is used in the output
     config, overriding the type of indentation in the input config (if any).
 
+* 'encoding': ``None``
+
+    By default **ConfigObj** does not decode the file/strings you pass it into
+    Unicode [#]_. If you want your config file as Unicode (keys and members)
+    you need to provide an encoding to decode the file with. This encoding will
+    also be used to encode the config file when writing.
+    
+    You can change the encoding attribute at any time.
+    
+    Any characters in your strings that can't be encoded with the specified
+    encoding will raise a ``UnicodeEncodeError``.
+    
+    .. note::
+    
+        ``UTF16`` encoded files will automatically be detected and decoded,
+        even if ``encoding`` is ``None``.
+        
+        This is because it is a 16-bit encoding, and ConfigObj will mangle it
+        (split characters on byte boundaries) if it parses it without decoding.
+
+* 'default_encoding': ``None``
+
+    When using the ``write`` method, **ConfigObj** uses the ``encoding``
+    attribute to encode the Unicode strings. If any members (or keys) have
+    been set as byte strings instead of Unicode, these must first be decoded
+    to Unicode before outputting in the specified encoding.
+
+    ``default_encoding``, if specified, is the encoding used to decode byte
+    strings in the **ConfigObj** before writing. If this is ``None``, then
+    the Python default encoding (``sys.defaultencoding`` - usually ascii) is
+    used.
+    
+    For most Western European users, a value of ``latin-1`` is sensible.
+    
+    ``default_encoding`` is *only* used if an ``encoding`` is specified.
+    
+    Any characters in byte-strings that can't be decoded using the
+    ``default_encoding`` will raise a ``UnicodeDecodeError``.
+
+
 Methods
 -------
 
@@ -409,9 +476,30 @@
 subclass of ``dict``, the builtin dictionary type. This means it also has
 **all** the normal dictionary methods.
 
-In addition, the *encode*, *decode*, *walk*, and *dict* methods of section may
-be useful. The sections and subsections are also instances of ``Section``. Read
-about Sections_ for details of all the methods.
+In addition, the following `Section Methods`_ may be useful :
+
+* *encode*
+* *decode*
+* *walk*
+* *merge*
+* *dict* 
+* *as_bool*
+* *as_float*
+* *as_int*
+
+Read about Sections_ for details of all the methods.
+
+.. hint::
+
+    The *merge* method of sections is a recursive update.
+    
+    You can use this to merge sections, or even whole ConfigObjs, into each
+    other.
+    
+    You would typically use this to create a default ConfigObj and then merge
+    in user settings. This way users only need to specify values that are
+    different from the default.
+    
 
 The public methods available on ConfigObj are :
 
@@ -421,19 +509,27 @@
 write
 ~~~~~
 
-This method takes no arguments [#]_. It writes the current ConfigObj. What that
-means depends on the ``filename`` attribute of the ConfigObj.
-
-filename
-    ConfigObj will write the configuration to file.
+::
+
+    write(file_object=None)
+
+This method writes the current ConfigObj and takes a single, optional argument
+[#]_.
+
+If you pass in a file like object to the ``write`` method, the config file will
+be written to this. (The only method of this object that is used is its
+``write`` method, so a ``StringIO`` instance, or any other file like object
+will work.)
+
+Otherwise, the behaviour of this method depends on the ``filename`` attribute
+of the ConfigObj.
+
+``filename``
+    ConfigObj will write the configuration to the file specified.
 
 ``None``
     ``write`` returns a list of lines. (Not ``'\n'`` terminated)
 
-``StringIO``
-    If ``filename`` is an object with a seek attribute, then the config file is
-    written to that object.
-
 First the 'initial_comment' is written, then the config file, followed by the
 'final_comment'. Comment lines and inline comments are written with each
 key/value.
@@ -441,6 +537,10 @@
 validate
 ~~~~~~~~
 
+::
+
+    validate(validator, preserve_errors=False)
+
 .. raw:: html
 
     {+coloring}
@@ -458,6 +558,10 @@
 
     {-coloring}
 
+The validate method uses the `validate 
+<http://www.voidspace.org.uk/python/validate.html>`__ module to do the
+validation.
+    
 This method validates the ConfigObj against the configspec. By doing type
 conversion as well, it can abstract away the config file altogether and present
 the config *data* to your application (in the types it expects it to be).
@@ -483,12 +587,13 @@
     
     See `ConfigObj for Data Persistence`_.
 
+
 Return Value
 ############
 
-The validate method either returns ``True`` (everything passed) or a dictionary 
-of ``True``/``False`` representing pass/fail. The dictionary follows the 
-structure of the ConfigObj
+By default, the validate method either returns ``True`` (everything passed) 
+or a dictionary of ``True``/``False`` representing pass/fail. The dictionary 
+follows the structure of the ConfigObj.
 
 If a whole section passes then it is replaced with the value ``True``. If a 
 whole section fails, then it is replaced with the value ``False``.
@@ -496,6 +601,18 @@
 If a value is missing, and there is no default in the check, then the check 
 automatically fails.
 
+The ``validate`` method takes an optional keyword argument ``preserve_errors``.
+If you set this to ``True``, instead of getting ``False`` for failed checks you
+get the actual error object from the **validate** module. This usually contains
+useful information about why the check failed.
+
+See the `flatten_errors`_ function for how to turn your results dictionary into
+a useful list of error messages.
+
+Even if ``preserve_errors`` is ``True``, missing keys or sections will still be
+represented by a ``False`` in the results dictionary.
+
+
 Mentioning Default Values
 #########################
 
@@ -536,6 +653,11 @@
     ``None``. This is because setting a value to a non-string raises an error
     if stringify is unset.
 
+The default value can be a list. See `List Values`_ for the way to do this.
+
+Writing invalid default values is a *guaranteed* way of confusing your users.
+Default values **must** pass the check.
+
 Mentioning Repeated Sections
 ############################
 
@@ -554,6 +676,7 @@
 
 See `Repeated Sections`_ for examples.
 
+
 Mentioning SimpleVal
 ####################
 
@@ -575,6 +698,7 @@
 
     {-coloring}
 
+    
 Attributes
 ----------
 
@@ -586,6 +710,9 @@
 * BOM
 * initial_comment
 * final_comment
+* list_values
+* encoding
+* default_encoding
 
 .. note::
 
@@ -598,7 +725,6 @@
 * raise_errors
 * create_empty
 * file_error
-* list_values
 
 interpolate
 ~~~~~~~~~~~
@@ -623,10 +749,13 @@
 ~~~
 
 If the initial config file *started* with the UTF8 Unicode signature (known
-slightly incorrectly as the {acro;BOM;Byte Order Mark}), then this value will
-be set to the UTF8 BOM. Otherwise it is ``None``.
-
-If it is set, then it is  written out by the ``write`` method.
+slightly incorrectly as the {acro;BOM;Byte Order Mark}), or the UTF16 BOM, then
+this attribute is set to ``True``. Otherwise it is ``False``.
+
+If it is set to ``True`` when ``write`` is called then, if ``encoding`` is set
+to ``None`` *or* to ``utf_8`` (and variants) a UTF BOM will be written.
+
+For UTF16 encodings, a BOM is *always* written.
 
 initial_comment
 ~~~~~~~~~~~~~~~
@@ -649,6 +778,52 @@
 The ``write`` method puts these lines after it finishes writing out the
 members.
 
+list_values
+~~~~~~~~~~~
+
+This attribute is ``True`` or ``False``. If set to ``False`` then values are
+not parsed for list values. In addition single line values are not unquoted.
+
+This allows you to do your own parsing of values. It exists primarily to
+support the reading of the configspec_ - but has other use cases.
+
+For example you could use the ``LineParser`` from the
+`listquote module <http://www.voidspace.org.uk/python/listquote.html#lineparser>`_ 
+to read values for nested lists.
+
+Single line values aren't quoted when writing - but multiline values are
+handled as normal.
+
+.. caution::
+
+    Because values aren't quoted, leading or trailing whitespace can be
+	lost.
+
+    This behaviour was changed in version 4.0.1.
+	
+	Prior to this, single line values might have been quoted; even with
+	``list_values=False``. This means that files written by **ConfigObj**
+	*could* now be incompatible - and need the quotes removing by hand.
+
+encoding
+~~~~~~~~
+
+This is the encoding used to encode the output, when you call ``write``. It
+must be a valid encoding `recognised by Python <http://docs.python.org/lib/standard-encodings.html>`_.
+
+If this value is ``None`` then no encoding is done when ``write`` is called.
+
+
+default_encoding
+~~~~~~~~~~~~~~~~
+
+If encoding is set, any byte-strings in your ConfigObj instance (keys or
+members) will first be decoded to unicode using the encoding specified by the
+``default_encoding`` attribute. This ensures that the output is in the encoding
+specified.
+
+If this value is ``None`` then ``sys.defaultencoding`` is used instead.
+
 The Config File Format
 ======================
 
@@ -943,13 +1118,13 @@
 Section Methods
 ---------------
 
-* 'dict'
+* **dict**
 
     This method takes no arguments. It returns a deep copy of the section as a
     dictionary. All subsections will also be dictionaries, and list values will
     be copies, rather than references to the original [#]_.
 
-* rename
+* **rename**
 
     ``rename(oldkey, newkey)``
 
@@ -958,22 +1133,52 @@
     It is mainly implemented for the ``encode`` and ``decode`` methods, which
     provide some Unicode support.
 
-* walk
+* **merge**
+
+    ``merge(indict)``
+    
+    This method is a *recursive update* method. It allows you to merge two
+    config files together.
+    
+    You would typically use this to create a default ConfigObj and then merge
+    in user settings. This way users only need to specify values that are
+    different from the default.
+    
+    For example :
+    
+    .. raw:: html
+    
+        {+coloring}
+        
+        # def_cfg contains your default config settings
+        # user_cfg contains the user settings
+        cfg = ConfigObj(def_cfg)
+        usr = ConfigObj(user_cfg)
+        #
+        cfg.merge(usr)
+        
+        """
+        cfg now contains a combination of the default settings and the user
+        settings.
+        
+        The user settings will have overwritten any of the default ones.
+        """
+    
+        {-coloring}
+    
+* **walk**
 
     This method can be used to transform values and names. See `walking a
     section`_ for examples and explanation.
 
-* decode
+* **decode**
 
     ``decode(encoding)``
 
     This method decodes names and values into Unicode objects, using the
     supplied encoding.
 
-    Because of the way ConfigObj reads files, config files should be in an
-    ASCII compatible encoding. See encodings_ for more details.
-
-* encode
+* **encode**
 
     ``encode(encoding)``
 
@@ -981,10 +1186,51 @@
 
     It encodes names and values using the supplied encoding. If any of your
     names/values are strings rather than Unicode, Python will have to do an
-    implicit decode first.
-
-    See encodings_ for more details.
-
+    implicit decode first. (This method uses ``sys.defaultencoding`` for
+    implicit decodes.)
+
+* **as_bool**
+
+    ``as_bool(key)``
+    
+    Returns ``True`` if the key contains a string that represents ``True``, or
+    is the ``True`` object.
+    
+    Returns ``False`` if the key contains a string that represents ``False``, 
+    or is the ``False`` object. 
+
+    Raises a ``ValueError`` if the key contains anything else.
+    
+    Strings that represent ``True`` are (not case sensitive) : ::
+    
+        true, yes, on, 1
+        
+    Strings that represent ``False`` are : ::
+    
+        false, no, off, 0
+    
+    .. note::
+    
+        In ConfigObj 4.1.0, this method was called ``istrue``. That method is
+        now deprecated and will issue a warning when used. It will go away
+        in the next release.
+        
+* **as_int**
+
+    ``as_int(key)``
+    
+    This returns the value contained in the specified key as an integer.
+    
+    It raises a ``ValueError`` if the conversion can't be done.
+
+* **as_float**
+
+    ``as_float(key)``
+    
+    This returns the value contained in the specified key as a float.
+    
+    It raises a ``ValueError`` if the conversion can't be done.
+    
 Walking a Section
 -----------------
 
@@ -1028,6 +1274,12 @@
 If you are using the return value from ``walk`` *and* ``call_on_sections``,
 note that walk discards the return value when it calls your function.
 
+.. caution::
+
+    You can use ``walk`` to transform the names of members of a section
+    but you mustn't add or delete members.
+
+
 Examples
 --------
 
@@ -1040,7 +1292,7 @@
 
 You can use this for transforming all values in your ConfigObj. For example
 you might like the nested lists from ConfigObj 3. This was provided by the
-listquote_ module [#]_. You could switch off the parsing for list values
+listquote_ module. You could switch off the parsing for list values
 (``list_values=False``) and use listquote to parse every value.
 
 Another thing you might want to do is use the Python escape codes in your
@@ -1116,6 +1368,59 @@
 
     {-coloring}
 
+Here's a simple example of using ``walk`` to transform names and values. One
+usecase of this would be to create a *standard* config file with placeholders
+for section and keynames. You can then use walk to create new config files
+and change values and member names :
+
+.. raw:: html
+
+    {+coloring}
+
+    # We use 'XXXX' as a placeholder
+    config = '''
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+    [XXXXsection1]
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+    [XXXXsection2]
+    XXXXkey1 = XXXXvalue1
+    XXXXkey2 = XXXXvalue2
+    XXXXkey3 = XXXXvalue3
+        [[XXXXsection1]]
+        XXXXkey1 = XXXXvalue1
+        XXXXkey2 = XXXXvalue2
+        XXXXkey3 = XXXXvalue3
+    '''.splitlines()
+    cfg = ConfigObj(config)
+    #
+    def transform(section, key):
+        val = section[key]
+        newkey = key.replace('XXXX', 'CLIENT1')
+        section.rename(key, newkey)
+        if isinstance(val, (tuple, list, dict)):
+            pass
+        else:
+            val = val.replace('XXXX', 'CLIENT1')
+            section[newkey] = val
+    #
+    cfg.walk(transform, call_on_sections=True)
+    print cfg
+    {'CLIENT1key1': 'CLIENT1value1', 'CLIENT1key2': 'CLIENT1value2', 
+    'CLIENT1key3': 'CLIENT1value3', 
+    'CLIENT1section1': {'CLIENT1key1': 'CLIENT1value1', 
+        'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3'}, 
+    'CLIENT1section2': {'CLIENT1key1': 'CLIENT1value1', 
+        'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3', 
+        'CLIENT1section1': {'CLIENT1key1': 'CLIENT1value1', 
+            'CLIENT1key2': 'CLIENT1value2', 'CLIENT1key3': 'CLIENT1value3'}}}
+            
+    {-coloring}
+
+
 Exceptions
 ==========
 
@@ -1127,7 +1432,7 @@
     raised.
 
 2) If you try to set a non-string key, or a non string value when
-    ``stringify=False`, a ``TypeError`` will be raised.
+    ``stringify=False``, a ``TypeError`` will be raised.
 
 3) A badly built config file will cause parsing errors.
 
@@ -1204,6 +1509,7 @@
     list of all the lines that caused errors may not be as useful as it sounds.
     {sm;:-(}.
 
+
 Validation
 ==========
 
@@ -1262,6 +1568,15 @@
 For full details on writing configspecs, please refer to the `validate.py
 documentation`_.
 
+.. important::
+
+    Your configspec is read by ConfigObj in the same way as a config file.
+    
+    That means you can do interpolation *within* your configspec.
+    
+    In order to allow this, checks in the 'DEFAULT' section (of the root level
+    of your configspec) are *not* used.
+
 .. _validate.py documentation: http://www.voidspace.org.uk/python/validate.html
 
 Type Conversion
@@ -1305,6 +1620,20 @@
 
     Note that ``default=None`` is case sensitive.
 
+
+List Values
+~~~~~~~~~~~
+
+It's possible that you will want to specify a list as a default value. To avoid
+confusing syntax with commas and quotes you use a list constructor to specify 
+that keyword arguments are lists. This includes the ``default`` value. This 
+makes checks look something like : ::
+
+    checkname(default=list('val1', 'val2', 'val3'))
+
+This works with all keyword arguments, but is most useful for default values.
+
+
 Repeated Sections
 -----------------
 
@@ -1404,6 +1733,7 @@
 
     {-coloring}
 
+
 Interpolation
 =============
 
@@ -1434,6 +1764,7 @@
 See `Validation and Interpolation`_. (This can only happen if validation
 has to *change* the value).
 
+
 Comments
 ========
 
@@ -1459,36 +1790,78 @@
 
 These comments are all written back out by the ``write`` method.
 
-Encodings
-=========
-
-ConfigObj 4 is designed to work with ASCII compatible encodings [#]_. If you
-need support for other character sets, then I suggest you use the UTF8
-encoding for your config files.
-
-By default ConfigObj leaves keys/members as encoded byte strings (ordinary
-strings). If you want to access the config file members as Unicode objects
-rather than strings, you can use the ``decode`` method. This takes an encoding
-as its argument and decodes all members and keys into Unicode. It will only
-work if *all* members are byte strings (or lists of strings) , so you should do
-it before calling ``validate``.
-
-If you want to turn the Unicode strings back into byte strings, you can call
-the ``encode`` method. This also takes an encoding as its argument and assumes
-*all* keys/members are Unicode.
-
-If you start working with Unicode strings, you may find you get
-``UnicodeDecodeError`` or ``UnicodeEncodeError`` in unexpected places. This is
-because you have forced Python to do an *implicit* encode or decode.
-
-Implicit decodes (and encodes) use the encoding returned by
-``sys.getdefaultencoding()``. This is *usually* ASCII. This means that if you
-have any non-ASCII characters, Python doesn't know how to treat them and will
-raise an error.
-
-This happens if you add a byte string to a Unicode string, compare a byte
-string to a Unicode string, print a Unicode string, or write it to a file. If
-you work with Unicode, you should do the appropriate encode or decode first.
+
+flatten_errors
+==============
+
+::
+
+    flatten_errors(cfg, res)
+
+Validation_ is a powerful way of checking that the values supplied by the user
+make sense.
+
+The validate_ method returns a results dictionary that represents pass or fail
+for each value. This doesn't give you any information about *why* the check
+failed.
+
+``flatten_errors`` is an example function that turns a results dictionary into
+a flat list, that only contains vaues that *failed*.
+
+``cfg`` is the ConfigObj instance being checked, ``res`` is the results
+dictionary returned by ``validate``.
+
+It returns a list of keys that failed. Each member of the list is a tuple : ::
+
+    ([list of sections...], key, result)
+
+If ``validate`` was called with ``preserve_errors=False`` (the default)
+then ``result`` will always be ``False``.
+
+*list of sections* is a flattened list of sections that the key was found
+in.
+
+If the section was missing then key will be ``None``.
+
+If the value (or section) was missing then ``result`` will be ``False``.
+
+If ``validate`` was called with ``preserve_errors=True`` and a value
+was present, but failed the check, then ``result`` will be the exception
+object returned. You can use this as a string that describes the failure.
+
+For example :
+
+    *The value "3" is of the wrong type*.
+
+
+Example Usage
+-------------
+
+The output from ``flatten_errors`` is a list of tuples.
+
+Here is an example of how you could present this information to the user.
+
+.. raw:: html
+
+    {+coloring}
+    
+    vtor = validate.Validator()
+    # ini is your config file - cs is the configspec
+    cfg = ConfigObj(ini, configspec=cs)
+    res = cfg.validate(vtor, preserve_errors=True)
+    for entry in flatten_errors(cfg, res):
+        # each entry is a tuple
+        section_list, key, error = entry
+        if key is not None:
+           section_list.append(key)
+        else:
+            section_list.append('[missing section]')
+        section_string = ', '.join(section_list)
+        if error == False:
+            error = 'Missing value or section.'
+        print section_string, ' = ', error
+
+    {-coloring}
 
 Backwards Compatibility
 =======================
@@ -1546,9 +1919,6 @@
 
 The ``newline``, ``force_return``, and ``default`` options have gone.
 
-The ``encoding`` and ``backup_encoding`` methods have gone, replaced with the
-``encode`` and ``decode`` methods.
-
 ``fileerror`` and ``createempty`` options have become ``file_error`` and
 ``create_empty``.
 
@@ -1580,6 +1950,7 @@
 .. _ConfigObj 3.3.1: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj3.zip
 .. _ConfigObj 3 Docs: http://www.voidspace.org.uk/python/configobj3.html
 
+
 CREDITS
 =======
 
@@ -1593,6 +1964,7 @@
 
 Thanks to others for input and bugfixes.
 
+
 LICENSE
 =======
 
@@ -1602,7 +1974,7 @@
 Of course if it does, let us know and we'll fix the problem so it doesn't
 happen to anyone else {sm;:-)}. ::
 
-    Copyright (c) 2004 & 2005, Michael Foord & Nicola Larosa
+    Copyright (c) 2004 - 2006, Michael Foord & Nicola Larosa
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -1636,7 +2008,8 @@
 
 You should also be able to find a copy of this license at : `BSD License`_
 
-.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+.. _BSD License: http://www.voidspace.org.uk/python/license.shtml
+
 
 TODO
 ====
@@ -1649,14 +2022,15 @@
 When initialising a section from a ConfigObj *or* an ``OrderedDictionary``
 we could preserve ordering.
 
-Add an *odict* method which returns an `OrderedDictionary``.
+Add an *odict* method which returns an ``OrderedDictionary``.
+
 
 ISSUES
 ======
 
 .. note::
 
-    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    Please file any bug reports to `Michael Foord`_ or the **ConfigObj**
     `Mailing List`_.
 
 You can't have a keyword with the same name as a section (in the same section).
@@ -1670,40 +2044,17 @@
 parent*: this means that checking the 'DEFAULT' sub-section in the *current
 section* is not necessarily logical ?
 
-In order to simplify Unicode support (which is possibly of limited value
-in a config file ?) I have removed automatic support, and added thev``encode``
-and ``decode`` methods. These can be used to transform keys and entries.
-Because the regex looks for specific values on inital parsing (i.e. the quotes
-and the equals signs) it can only read ASCII compatible encodings. For Unicode
-I suggest ``UTF8``, which is ASCII compatible.
-
-.. note::
-
-    There is no reason why you shouldn't decode your config file into Unicode
-    before passing them to ConfigObj (as a list of lines). This should give you
-    Unicode keys and values.
-
 Does it matter that we don't support the ':' divider, which is supported by
 ``ConfigParser`` ?
 
-Following error with "list_values=False" : ::
-
-    >>> a = ["a='hello', 'goodbye'"]
-    >>>
-    >>> c(a, list_values=False)
-    {'a': "hello', 'goodbye"}
-
-The regular expression correctly removes the value - ``"'hello', 'goodbye'"`` -
-and then unquote just removes the front and back quotes (called by
-``_handle_value``). What should we do ?? We *ought* to raise an exception - 
-because if lists are off it's an invalid value. This is not desired if you want 
-to do your own list processing e.g. using listquote for nested lists. It would
-be *better* in this case not to unquote. Raising an exception would require a
-new regex.
-
 String interpolation and validation don't play well together. See 
 `Validation and Interpolation`_.
 
+Validation is no longer done on the 'DEFAULT' section (on the root level). This
+allows interpolation from within your configspec - but also prevents you
+validating the 'DEFAULT' section.
+
+
 CHANGELOG
 =========
 
@@ -1711,6 +2062,78 @@
 From version 4 it lists all releases and changes. *More* data on individual
 changes may be found in the source code.
 
+2006/02/16 - Version 4.2.0
+--------------------------
+
+Removed ``BOM_UTF8`` from ``__all__``.
+
+The ``BOM`` attribute has become a boolean. (Defaults to ``False``.) It is
+*only* ``True`` for the ``UTF16`` encoding.
+
+File like objects no longer need a ``seek`` attribute.
+
+Full unicode support added. New options/attributes ``encoding``,
+``default_encoding``.
+
+ConfigObj no longer keeps a reference to file like objects. Instead the
+``write`` method takes a file like object as an optional argument. (Which
+will be used in preference of the ``filename`` attribute if that exists as
+well.)
+
+utf16 files decoded to unicode.
+
+If ``BOM`` is ``True``, but no encoding specified, then the utf8 BOM is
+written out at the start of the file. (It will normally only be ``True`` if
+the utf8 BOM was found when the file was read.)
+
+File paths are *not* converted to absolute paths, relative paths will
+remain relative as the ``filename`` attribute.
+
+Fixed bug where ``final_comment`` wasn't returned if ``write`` is returning
+a list of lines.
+
+Deprecated ``istrue``, replaced it with ``as_bool``.
+
+Added ``as_int`` and ``as_float``.
+
+2005/12/14 - Version 4.1.0
+--------------------------
+
+Added ``merge``, a recursive update.
+
+Added ``preserve_errors`` to ``validate`` and the ``flatten_errors``
+example function.
+
+Thanks to Matthew Brett for suggestions and helping me iron out bugs.
+    
+Fixed bug where a config file is *all* comment, the comment will now be
+``initial_comment`` rather than ``final_comment``.
+
+Validation no longer done on the 'DEFAULT' section (only in the root level).
+This allows interpolation in configspecs.
+
+Also use the new list syntax in validate_ 0.2.1. (For configspecs).
+
+2005/12/02 - Version 4.0.2
+--------------------------
+
+Fixed bug in ``create_empty``. Thanks to Paul Jimenez for the report.
+
+
+2005/11/05 - Version 4.0.1
+--------------------------
+
+Fixed bug in ``Section.walk`` when transforming names as well as values.
+
+Added the ``istrue`` method. (Fetches the boolean equivalent of a string
+value).
+
+Fixed ``list_values=False`` - they are now only quoted/unquoted if they
+are multiline values.
+
+List values are written as ``item, item`` rather than ``item,item``.
+
+
 2005/10/17 - Version 4.0.0
 --------------------------
 
@@ -1878,19 +2301,18 @@
 Footnotes
 =========
 
-.. [#] The core parser is now based on regular expressions, so it's a lot
-    faster.
-
-.. [#] 134 of them, at the time of writing.
+.. [#] 253 of them, at the time of writing.
 
 .. [#] And if you discover any bugs, let us know. We'll fix them quickly.
 
 .. [#] If you specify a filename that doesn't exist, ConfigObj will assume you
     are creating a new one. See the *create_empty* and *file_error* options_.
 
-.. [#] They can be byte strings ('ordinary' strings) or Unicode. If they are
-    Unicode then ConfigObj will have to do an implicit encode before writing.
-    See the encodings_ section for more details.
+.. [#] They can be byte strings ('ordinary' strings) or Unicode.
+
+.. [#] This is a change in ConfigObj 4.2.0. Note that ConfigObj doesn't call
+    the seek method of any file like object you pass in. You may want to call
+    ``file_object.seek(0)`` yourself, first.
 
 .. [#] Except we don't support the RFC822 style line continuations, nor ':' as
     a divider.
@@ -1901,26 +2323,26 @@
 
 .. [#] A side effect of this is that it enables you to copy a ConfigObj :
 
-.. raw:: html
-
-    {+coloring}
-
-    # only copies members
-    # not attributes/comments
-    config2 = ConfigObj(config1)
-
-    {-coloring}
-
-..
+    .. raw:: html
+    
+        {+coloring}
+    
+        # only copies members
+        # not attributes/comments
+        config2 = ConfigObj(config1)
+    
+        {-coloring}
 
     The order of values and sections will not be preserved, though.
 
+.. [#] The exception is if it detects a ``UTF16`` encoded file which it
+    must decode before parsing.
+     
 .. [#] Other than lists of strings.
 
-.. [#] The method signature in the API docs will show that it does in fact
-    take one argument: the section to be written. This is because the
-    ``write`` method is called recursively. Using this argument *forces* write
-    to return a list of lines, so it's probably not very useful to you.
+.. [#] The method signature in the API docs will show that this method takes
+    two arguments. The second is the section to be written. This is because the
+    ``write`` method is called recursively.
 
 .. [#] The dict method doesn't actually use the deepcopy mechanism. This means
     if you add nested lists (etc) to your ConfigObj, then the dictionary
@@ -1931,9 +2353,6 @@
     change the value by setting ``section[key] = newval``. It also gives you
     access to the *rename* method of the section.
 
-.. [#] A current bug in unquoting when ``list_values=False`` currently makes
-    this impossible. We are considering the best way to fix this.
-
 .. [#] Minimum required version of *validate.py* 0.2.0 .
 
 .. [#] There's nothing to stop you decoding the whole config file to Unicode
@@ -1959,7 +2378,7 @@
         <br />
         <a href="http://www.python.org">
             <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
-        </a>
+        </a><br /><br />
         <a href="http://www.opensource.org">
             <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
             <br /><strong>Certified Open Source</strong>
@@ -1972,10 +2391,9 @@
                 <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
             </a>
         </noscript>
-        <br />
     </div>
 
 .. _listquote: http://www.voidspace.org.uk/python/modules.shtml#listquote
-.. _Michael Foord: http://www.voidspace.org.uk
+.. _Michael Foord: http://www.voidspace.org.uk/index2.shtml
 .. _Nicola Larosa: http://www.teknico.net
 .. _Mark Andrews: http://www.la-la.com

=== modified file 'bzrlib/util/configobj/docs/validate.txt'
--- bzrlib/util/configobj/docs/validate.txt	2005-10-18 18:48:27 +0000
+++ bzrlib/util/configobj/docs/validate.txt	2006-02-21 02:26:30 +0000
@@ -8,8 +8,8 @@
 
 
 :Authors: `Michael Foord`_, `Nicola Larosa`_, `Mark Andrews`_
-:Version: Validate 0.2.0
-:Date: 2005/08/25
+:Version: Validate 0.2.1
+:Date: 2005/12/16
 :Homepage: `Validate Homepage`_
 :License: `BSD License`_
 :Support: `Mailing List`_
@@ -20,7 +20,7 @@
 .. _Mark Andrews: mark@la-la.com
 .. _This Document:
 .. _Validate Homepage: http://www.voidspace.org.uk/python/validate.html
-.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+.. _BSD License: http://www.voidspace.org.uk/python/license.shtml
 
 
 .. contents:: Validate Manual
@@ -56,7 +56,7 @@
 into every validator. Additional checks are easy to write: they can be provided
 when the ``Validator`` is instantiated, or added afterwards.
 
-Validate was primarily written to support ConfigObj_, but was designed to be 
+Validate was primarily written to support ConfigObj_, but is designed to be 
 applicable to many other situations.
 
 For support and bug reports please use the ConfigObj `Mailing List`_.
@@ -66,7 +66,7 @@
 Downloading
 ===========
 
-The current version is **0.2.0**, dated 25th August 2005. 
+The current version is **0.2.1**, dated 16th December 2005. 
 
 You can get obtain validate in the following ways :
 
@@ -76,7 +76,7 @@
 * validate.py_ from Voidspace
 
 * configobj.zip from Voidspace - See the homepage of ConfigObj_ for the latest 
-  version and downlaod links.
+  version and download links.
 
     This contains validate.py and `this document`_. (As well as ConfigObj_ and 
     the ConfigObj documentation).
@@ -101,7 +101,7 @@
 Python Projects`_.
 
 .. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
-.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0b4.zip
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.1.0.zip
 .. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
 .. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
 .. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4/
@@ -203,8 +203,8 @@
     from validate import Validator
     #
     vtor = Validator()
-    newval1 = vtor.check(value1, 'integer')
-    newval2 = vtor.check(value2, 'bool')
+    newval1 = vtor.check('integer', value1)
+    newval2 = vtor.check('bool', value2)
     # etc ...
 
     {-coloring}
@@ -215,6 +215,7 @@
     them in ``try...except`` blocks. Better still,  use ConfigObj for a higher
     level interface.
 
+
 Using Validator
 ===============
 
@@ -332,6 +333,7 @@
     Although the value can be a string, if it represents a list it should
     already have been turned into a list of strings.
 
+
 Default Values
 ~~~~~~~~~~~~~~
 
@@ -365,6 +367,20 @@
 that this check contains no useful value when missing, i.e. the value is
 optional, and may be omitted without harm.
 
+
+List Values
+~~~~~~~~~~~
+
+It's possible that you would like your default value to be a list. It's even
+possible that you will write your own check functions - and would like to pass
+them keyword arguments as lists from within the check.
+
+To avoid confusing syntax with commas and quotes you use a list constructor to
+specify that keyword arguments are lists. This includes the ``default`` value.
+This makes checks look something like : ::
+
+    checkname(default=list('val1', 'val2', 'val3'))
+
 Validator Exceptions
 ====================
 
@@ -539,7 +555,7 @@
 
 * A regex check function ?
 * A timestamp check function ? (Using the ``parse`` function from ``DateUtil``).
-* Allow triple quotes ? (getting a bit heavy for a single regex)
+* Allow triple quotes ? (getting a bit heavy for a regex)
 
 ISSUES
 ======
@@ -549,15 +565,25 @@
     Please file any bug reports to `Michael Foord`_ or the ConfigObj
     `Mailing List`_.
 
-Lists passed as function arguments need additional quotes. Ugly, could do
-with fixing this.
-
 If we could pull tuples out of arguments, it would be easier
 to specify arguments for 'mixed_lists'.
 
 CHANGELOG
 =========
 
+2005/12/16      Version 0.2.1
+-----------------------------
+
+Fixed bug so we can handle keyword argument values with commas.
+
+We now use a list constructor for passing list values to keyword arguments
+(including ``default``) : ::
+
+    default=list("val", "val", "val")
+
+Added the ``_test`` test. {sm;:-)}
+
+Moved a function call outside a try...except block.
 
 2005/08/18      Version 0.2.0
 -----------------------------

=== modified file 'bzrlib/util/configobj/validate.py'
--- bzrlib/util/configobj/validate.py	2005-10-18 18:48:27 +0000
+++ bzrlib/util/configobj/validate.py	2006-02-21 02:26:30 +0000
@@ -6,7 +6,7 @@
 #         nico AT tekNico DOT net
 
 # This software is licensed under the terms of the BSD license.
-# http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+# http://www.voidspace.org.uk/python/license.shtml
 # Basically you're free to copy, modify, distribute and relicense it,
 # So long as you keep a copy of the license with it.
 
@@ -112,11 +112,21 @@
                 You specify this check with : ::
     
                   option('option 1', 'option 2', 'option 3')
+    
+    You can supply a default value (returned if no value is supplied)
+    using the default keyword argument.
+    
+    You specify a list argument for default using a list constructor syntax in
+    the check : ::
+    
+        checkname(arg1, arg2, default=list('val 1', 'val 2', 'val 3'))
+    
+    A badly formatted set of arguments will raise a ``VdtParamError``.
 """
 
 __docformat__ = "restructuredtext en"
 
-__version__ = '0.2.0'
+__version__ = '0.2.1'
 
 __revision__ = '$Id: validate.py 123 2005-09-08 08:54:28Z fuzzyman $'
 
@@ -159,6 +169,84 @@
 import re
 StringTypes = (str, unicode)
 
+
+_list_arg = re.compile(r'''
+    (?:
+        ([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*list\(
+            (
+                (?:
+                    \s*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s\)][^,\)]*?)  # unquoted
+                    )
+                    \s*,\s*
+                )*
+                (?:
+                    (?:".*?")|              # double quotes
+                    (?:'.*?')|              # single quotes
+                    (?:[^'",\s\)][^,\)]*?)  # unquoted
+                )?                          # last one
+            )
+        \)
+    )
+''', re.VERBOSE)    # two groups
+
+_list_members = re.compile(r'''
+    (
+        (?:".*?")|              # double quotes
+        (?:'.*?')|              # single quotes
+        (?:[^'",\s=][^,=]*?)       # unquoted
+    )
+    (?:
+    (?:\s*,\s*)|(?:\s*$)            # comma
+    )
+''', re.VERBOSE)    # one group
+
+_paramstring = r'''
+    (?:
+        (
+            (?:
+                [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*list\(
+                    (?:
+                        \s*
+                        (?:
+                            (?:".*?")|              # double quotes
+                            (?:'.*?')|              # single quotes
+                            (?:[^'",\s\)][^,\)]*?)       # unquoted
+                        )
+                        \s*,\s*
+                    )*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s\)][^,\)]*?)       # unquoted
+                    )?                              # last one
+                \)
+            )|
+            (?:
+                (?:".*?")|              # double quotes
+                (?:'.*?')|              # single quotes
+                (?:[^'",\s=][^,=]*?)|       # unquoted
+                (?:                         # keyword argument
+                    [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*
+                    (?:
+                        (?:".*?")|              # double quotes
+                        (?:'.*?')|              # single quotes
+                        (?:[^'",\s=][^,=]*?)       # unquoted
+                    )
+                )
+            )
+        )
+        (?:
+            (?:\s*,\s*)|(?:\s*$)            # comma
+        )
+    )
+    '''
+
+_matchstring = '^%s*' % _paramstring
+
 # Python pre 2.2.1 doesn't have bool
 try:
     bool
@@ -433,13 +521,25 @@
         http://www.voidspace.org.uk/python/configobj.html
     """
 
-    # this regex pulls values out of a comma separated line
-    _paramfinder = re.compile(r'''(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)''')
-    # this regex is used for finding keyword arguments
-    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$')
     # this regex does the initial parsing of the checks
     _func_re = re.compile(r'(.+?)\((.*)\)')
 
+    # this regex takes apart keyword arguments
+    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$')
+
+
+    # this regex finds keyword=list(....) type values
+    _list_arg = _list_arg
+
+    # this regex takes individual values out of lists - in one pass
+    _list_members = _list_members
+
+    # These regexes check a set of arguments for validity
+    # and then pull the members out
+    _paramfinder = re.compile(_paramstring, re.VERBOSE)
+    _matchfinder = re.compile(_matchstring, re.VERBOSE)
+
+
     def __init__(self, functions=None):
         """
         >>> vtri = Validator()
@@ -487,23 +587,29 @@
         fun_match = self._func_re.match(check)
         if fun_match:
             fun_name = fun_match.group(1)
+            arg_string = fun_match.group(2)
+            arg_match = self._matchfinder.match(arg_string)
+            if arg_match is None:
+                # Bad syntax
+                raise VdtParamError
             fun_args = []
             fun_kwargs = {}
             # pull out args of group 2
-            for arg in self._paramfinder.findall(fun_match.group(2)):
+            for arg in self._paramfinder.findall(arg_string):
                 # args may need whitespace removing (before removing quotes)
                 arg = arg.strip()
+                listmatch = self._list_arg.match(arg)
+                if listmatch:
+                    key, val = self._list_handle(listmatch)
+                    fun_kwargs[key] = val
+                    continue
                 keymatch = self._key_arg.match(arg)
                 if keymatch:
-                    val = keymatch.group(2)
-                    if (val[0] in ("'", '"')) and (val[0] == val[-1]):
-                        val = val[1:-1]
+                    val = self._unquote(keymatch.group(2))
                     fun_kwargs[keymatch.group(1)] = val
                     continue
                 #
-                if (arg[0] in ("'", '"')) and (arg[0] == arg[-1]):
-                    arg = arg[1:-1]
-                fun_args.append(arg)
+                fun_args.append(self._unquote(arg))
         else:
             # allows for function names without (args)
             (fun_name, fun_args, fun_kwargs) = (check, (), {})
@@ -524,9 +630,28 @@
         except KeyError:
             pass
         try:
-            return self.functions[fun_name](value, *fun_args, **fun_kwargs)
+            fun = self.functions[fun_name]
         except KeyError:
             raise VdtUnknownCheckError(fun_name)
+        else:
+##            print fun_args
+##            print fun_kwargs
+            return fun(value, *fun_args, **fun_kwargs)
+
+    def _unquote(self, val):
+        """Unquote a value if necessary."""
+        if (len(val) > 2) and (val[0] in ("'", '"')) and (val[0] == val[-1]):
+            val = val[1:-1]
+        return val
+
+    def _list_handle(self, listmatch):
+        """Take apart a ``keyword=list('val, 'val')`` type string."""
+        out = []
+        name = listmatch.group(1)
+        args = listmatch.group(2)
+        for arg in self._list_members.findall(args):
+            out.append(self._unquote(arg))
+        return name, out
 
     def _pass(self, value):
         """
@@ -1059,6 +1184,47 @@
         raise VdtValueError(value)
     return value
 
+def _test(value, *args, **keywargs):
+    """
+    A function that exists for test purposes.
+    
+    >>> checks = [
+    ...     '3, 6, min=1, max=3, test=list(a, b, c)',
+    ...     '3',
+    ...     '3, 6',
+    ...     '3,',
+    ...     'min=1, test="a b c"',
+    ...     'min=5, test="a, b, c"',
+    ...     'min=1, max=3, test="a, b, c"',
+    ...     'min=-100, test=-99',
+    ...     'min=1, max=3',
+    ...     '3, 6, test="36"',
+    ...     '3, 6, test="a, b, c"',
+    ...     '3, max=3, test=list("a", "b", "c")',
+    ...     '''3, max=3, test=list("'a'", 'b', "x=(c)")''',
+    ...     "test='x=fish(3)'",
+    ...    ]
+    >>> v = Validator({'test': _test})
+    >>> for entry in checks:
+    ...     print v.check(('test(%s)' % entry), 3)
+    (3, ('3', '6'), {'test': ['a', 'b', 'c'], 'max': '3', 'min': '1'})
+    (3, ('3',), {})
+    (3, ('3', '6'), {})
+    (3, ('3',), {})
+    (3, (), {'test': 'a b c', 'min': '1'})
+    (3, (), {'test': 'a, b, c', 'min': '5'})
+    (3, (), {'test': 'a, b, c', 'max': '3', 'min': '1'})
+    (3, (), {'test': '-99', 'min': '-100'})
+    (3, (), {'max': '3', 'min': '1'})
+    (3, ('3', '6'), {'test': '36'})
+    (3, ('3', '6'), {'test': 'a, b, c'})
+    (3, ('3',), {'test': ['a', 'b', 'c'], 'max': '3'})
+    (3, ('3',), {'test': ["'a'", 'b', 'x=(c)'], 'max': '3'})
+    (3, (), {'test': 'x=fish(3)'})
+    """
+    return (value, args, keywargs)
+
+
 if __name__ == '__main__':
     # run the code tests in doctest format
     import doctest
@@ -1081,15 +1247,31 @@
     ISSUES
     ======
     
-    Lists passed as function arguments need additional quotes. Ugly, could do
-    with fixing this.
-    
     If we could pull tuples out of arguments, it would be easier
     to specify arguments for 'mixed_lists'.
     
     CHANGELOG
     =========
     
+    2005/12/16
+    ----------
+    
+    Fixed bug so we can handle keyword argument values with commas.
+    
+    We now use a list constructor for passing list values to keyword arguments
+    (including ``default``) : ::
+    
+        default=list("val", "val", "val")
+    
+    Added the ``_test`` test. {sm;:-)}
+    
+    0.2.1
+    
+    2005/12/12
+    ----------
+    
+    Moved a function call outside a try...except block.
+    
     2005/08/25
     ----------
     

